-- settings
randomColor = true -- whether or not a new box's color is randomly generated
boxColor = { 200, 0, 0 } -- if not random, this color will be used
minSize = 10 -- minimum box size
maxSize = 50 -- maximum box size
minMass = 5 -- minimum box mass
maxMass = 20 -- maximum box mass
motionFrames = 10 -- number of frames to store for motion; must be at least 1 (the current frame)
alphaMultiplier = 50 -- alpha of each motion is determined by: frameNumber * (1 / motionFrames * alphaMultiplier)
gravity = 500 -- gravity amount for the y-axis
vsync = true

function math.clamp(input, mini, maxi)
	if input < mini then
		input = mini
	elseif input > maxi then
		input = maxi
	end
	return input
end


function love.load()
  paused = false
  world = love.physics.newWorld(0, gravity)
  boxes = {}
  
  -- note: this code runs under the assumption that the window 800x600
  obstacles = {
    createObstacle(50, 100, 100, 20),
    createObstacle(200, 350, 200, 50),
    createObstacle(700, 50, 40, 200),
    createObstacle(450, 400, 100, 100),
    createObstacle(370, 100, 150, 100),
    
    -- walls
    createObstacle(0, 600, 800, 5), -- bottom wall
    createObstacle(0, -5, 800, 5), -- top wall
    createObstacle(-5, 0, 5, 600), -- left wall
    createObstacle(800, 0, 5, 600) -- right wall
  }
  
  canvas = love.graphics.newCanvas()
end

function love.update(dt2)
  dt = dt2
  if paused then return end
  world:update(dt)
  
  local xForce = 0
  local yForce = 0
  if love.keyboard.isDown("left") then xForce = xForce - 1000 end
  if love.keyboard.isDown("right") then xForce = xForce + 1000 end
  if love.keyboard.isDown("up") then yForce = yForce - 2000 end
  if love.keyboard.isDown("down") then yForce = yForce + 1000 end
  
  if xForce ~= 0 or yForce ~= 0 then
    for _, v in ipairs(boxes) do
      v.body:applyForce(xForce, yForce)
    end
  end
end

function love.draw()
  if not paused then
    love.graphics.setCanvas(canvas)
    love.graphics.setColor(0, 0, 0, math.clamp(dt*15*255, 4, 255))
    --Makes the transparency low as it adds to the canvas.
    love.graphics.rectangle('fill', 0, 0, 800,600)
    --Adds a background so the trails don't stick.
    love.graphics.setColor(255,255,255)
    for _, v in ipairs(obstacles) do
      love.graphics.rectangle("fill", v.x, v.y, v.width, v.height)
    end
    
    for _, v in ipairs(boxes) do
      love.graphics.setColor(v.color)
      love.graphics.polygon("fill", v.body:getWorldPoints(v.shape:getPoints()))
    end
    love.graphics.setCanvas()
  end
  love.graphics.setColor(255,255,255,255)
  love.graphics.draw(canvas, 0, 0)
  love.graphics.print(love.timer.getFPS(), 0, 0)
  if vsync then love.graphics.print('vsync = true', 0, 10) else love.graphics.print('vsync = false', 0, 10) end
end






function love.keypressed(key)
  if key == "p" then paused = not paused end
  if key == 'v' then vsync = not vsync; love.graphics.setMode(800, 600, false, vsync) end  
end

function love.mousepressed(x, y, button)
  if button == "l" then
    boxes[#boxes + 1] = createBox(x, y)
  end
end

function createObstacle(x, y, width, height)
  local o = {}
  o.x = x
  o.y = y
  o.width = width
  o.height = height
  o.body = love.physics.newBody(world, x + width / 2, y + height / 2, "static")
  o.shape = love.physics.newRectangleShape(width, height)
  o.fixture = love.physics.newFixture(o.body, o.shape)
  return o
end

function createBox(mx, my)
  local b = {}
  b.width = math.random(minSize, maxSize)
  b.height = math.random(minSize, maxSize)
  b.body = love.physics.newBody(world, mx, my, "dynamic")
  b.body:setMass(math.random(minMass, maxMass))
  b.shape = love.physics.newRectangleShape(b.width, b.height)
  b.fixture = love.physics.newFixture(b.body, b.shape)
  
  if randomColor then
    b.color = { math.random(255), math.random(255), math.random(255)}
  else
    b.color = boxColor -- not copied on purpose
  end
  
  return b
end